# mandayejs - saml reverse proxy
# Copyright (C) 2015  Entr'ouvert
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django import forms


class FormFactory(forms.Form):
    def __init__(self, *args, **kwargs): 
        fields = kwargs.pop('locators', None)
        super(FormFactory, self).__init__(*args, **kwargs)
        
        if fields : 
            for field in fields :
                if field['kind'] == 'string':
                    self.fields[field['name']] = forms.CharField(
                            label=field['label'], 
                            max_length=32, 
                            help_text=field['help']
                    )
                elif field['kind'] == 'password':
                    self.fields[field['name']] = forms.CharField(
                            label=field['label'], 
                            widget=forms.PasswordInput(), 
                            help_text=field['help']
                    )
                else:
                    self.fields[field['name']] = forms.DateField(
                            label=field['label'], 
                            help_text=field['help']
                    )

    def is_valid(self,):
        is_valid = super(FormFactory, self).is_valid()
        if not is_valid:
            return is_valid

        if self.data == {}:
            return False

        return is_valid

