from __future__ import absolute_import

import json
import ldif

from django.core.management.base import BaseCommand, CommandError
from django.db import IntegrityError
from django.contrib.auth import get_user_model

from mandayejs.mandaye.models import UserCredentials  


class Command(BaseCommand):
    args = '<ldif_file>'
    help = 'Migrate users from ldap'

    def handle(self, *args, **kwargs):

        if len(args) < 1:
            self.stdout.write('An input file is required')
            return False
        for ldif_file in args:
            data = self.get_ldif_data(ldif_file)
            data = [ d[1] for d in data]
            self.migrate(data)

    def get_ldif_data(self, filename):
        with open(filename, 'rb') as fd:
            return ldif.ParseLDIF(fd)

    def migrate(self, parsed_data):
        User = get_user_model()
        for data in parsed_data:
            data = { k : ''.join(v) for k,v in data.items()}
            try:
                user = User(username=data.get('idpUniqueID'), last_name=data.get('spLogin'))
                user.save()
                uc = UserCredentials(user=user, locators=json.loads(data.get('spPostValues')))
                uc.save()
                self.stdout.write('{idpUniqueID} imported'.format(**data))
            except (IntegrityError,) as e:
                continue

